//
//  kjots
//
//  Copyright (C) 1997 Christoph Neerfeld
//  Copyright (C) 2002, 2003 Aaron J. Seigo
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#include <qbuttongroup.h>
#include <qcursor.h>
#include <qobjectlist.h>
#include <qdatetime.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qtimer.h>

#include <kaccel.h>
#include <kaction.h>
#include <kstdaccel.h>
#include <kstdaction.h>
#include <kapplication.h>
#include <kedittoolbar.h>
#include <kfiledialog.h>
#include <kdebug.h>
#include <kkeydialog.h>
#include <klineeditdlg.h>
#include <klocale.h>
#include <kmenubar.h>
#include <kmessagebox.h>
#include <kopenwith.h>
#include <kpopupmenu.h>
#include <krun.h>
#include <kstandarddirs.h>
#include <kstringhandler.h>
#include <kurl.h>
#include <keditcl.h>
#include <kprinter.h>
#include <kglobal.h>

#include "KJotsMain.h"
#include "optiondialog.h"


#define KJOTSVERSION "0.5"

const unsigned int HOT_LIST_SIZE = 8;
const unsigned int BUTTON_WIDTH = 56;

//----------------------------------------------------------------------
// KJotsEntry
//----------------------------------------------------------------------

class KJotsEntry : public KListViewItem
{
public:

    KJotsEntry(KListView* parent, QString subj, QListViewItem* after = 0)
        : KListViewItem(parent, after)
    {
        setDragEnabled(true);
        setDropEnabled(true);
        setRenameEnabled(0, true);
        setSubject(subj);
    }

    ~KJotsEntry()
    {
    }

    void startRename(int col)
    {
        if (col == 0)
        {
            QListViewItem::setText(0, m_subject);
        }

        QListViewItem::startRename(col);
    }

    void editSubject()
    {
        startRename(0);
    }

    void setSubject(QString subj)
    {
        setText(0, subj);
    }

    QString subject()
    {
        return m_subject;
    }

    void setText(int column, const QString& text)
    {
        if (column == 0)
        {
            m_subject = text;

            if (text.isNull() || text.length() < 1)
            {
                QListViewItem::setText(0, i18n("Untitled"));
            }
            else
            {
                QListViewItem::setText(0,KStringHandler::rEmSqueeze(text,
                                                   listView()->fontMetrics(),
                                                   listView()->columnWidth(0)));
            }
        }
        else
        {
            QListViewItem::setText(column, text);
        }
    }

    void setBody(const QString& text)
    {
        m_text = text;
    }

    QString body()
    {
        return m_text;
    }

private:
    QString   m_text;
    QString   m_subject;
};
/*
class KJotsListView : public KListView
{
project:
    QDragObject* dragObject
}*/

//----------------------------------------------------------------------
// MYMULTIEDIT
//----------------------------------------------------------------------
MyMultiEdit::MyMultiEdit (QWidget* parent, const char* name)
  : KTextEdit(parent, name),
   mSearchDialog(0),
   mReplaceDialog(0)
{
  setTextFormat(Qt::RichText);
  setWordWrap(QTextEdit::WidgetWidth);
  setLinkUnderline(true);
  web_menu = new KPopupMenu(this);;
  web_menu->insertItem(i18n("Open URL"), this, SLOT(openUrl()) );
}

MyMultiEdit::~MyMultiEdit ()
{
  delete mReplaceDialog;
}

void MyMultiEdit::search()
{
    if (!mSearchDialog)
    {
        mSearchDialog = new KEdFind(this);
        connect(mSearchDialog, SIGNAL(search()), this, SLOT(slotSearch()));
    }
    mSearchDialog->show();
}

void MyMultiEdit::repeatSearch()
{
    if (mSearchFor.isNull())
    {
        return;
    }

    int para = 0;
    int index = 0;
    getCursorPosition(&para, &index);
    removeSelection();

    if (find(mSearchFor, mSearchDialog->case_sensitive(), false, !mSearchDialog->get_direction(), &para, &index))
    {
        setSelection(para, index, para, index + mSearchFor.length());
    }
    else
    {
        if (KMessageBox::questionYesNo(this, i18n("End of page reached.\n\nContinue from beginning of page?")) == KMessageBox::Yes)
        {
            setCursorPosition(0, 0);
            repeatSearch();
        }
    }
 }

void MyMultiEdit::slotSearch()
{
    int para = 0;
    int index = 0;
    getCursorPosition(&para, &index);
    mSearchFor = mSearchDialog->getText();
    removeSelection();

    if (find(mSearchFor, mSearchDialog->case_sensitive(), false, !mSearchDialog->get_direction(), &para, &index))
    {
        emit findSuccessful();
        setSelection(para, index, para, index + mSearchFor.length());
    }
    else
    {
        if (KMessageBox::questionYesNo(this, i18n("End of page reached.\n\nContinue from beginning of page?")) == KMessageBox::Yes)
        {
            setCursorPosition(0, 0);
            slotSearch();
        }
        else
        {
            mSearchDialog->hide();
        }
    }
}

void MyMultiEdit::mousePressEvent( QMouseEvent *e )
{
  if(e->button() == RightButton &&
     hasSelectedText())
  {
      KURL url(selectedText());

      if(url.isValid())
      {
         web_menu->popup(QCursor::pos());
          return;
      }
  }

  KTextEdit::mousePressEvent(e);
}

void MyMultiEdit::openUrl()
{
  if (hasSelectedText())
  {
    KURL url(selectedText());
    if(url.isValid())
    {
      KFileOpenWithHandler fowh;
      new KRun(url);
    }
  }
}

void MyMultiEdit::print(QString title)
{
    KPrinter printer;
    printer.setDocName(title);
    printer.setFullPage(false);
    printer.setCreator("KJots");

    if (printer.setup(this))
    {
        QFont printFont = font();
        QPainter painter( &printer );
        QPaintDeviceMetrics metrics( &printer );
        int y = 0;
        int maxWidth = metrics.width();
        int maxHeight = metrics.height();
        QString currentParagraph;

        for (int paragraphCount = 0; paragraphCount < paragraphs(); ++paragraphCount )
        {
            currentParagraph = text(paragraphCount);
            QRect r = painter.boundingRect(0, y, maxWidth, maxHeight,
                                           QPainter::ExpandTabs | QPainter::WordBreak,
                                           currentParagraph);

            if ((y + r.height()) > maxHeight)
            {
                printer.newPage();
                y = 0;
            }

            painter.drawText(0, y, maxWidth, maxHeight - y,
                             QPainter::ExpandTabs | QPainter::WordBreak,
                             currentParagraph);
            y += r.height();
        }
        painter.end();
    }
}

//
// KMyListView
// 

class MyListView:public KListView
{
public:
	MyListView(QWidget*,const char*);
   	
protected:
	void resizeEvent(QResizeEvent*);
};

MyListView::MyListView(QWidget* parent=0,const char* name=0)
	:KListView(parent,name)
{

}

void MyListView::resizeEvent(QResizeEvent* e)
{
	KListView::resizeEvent(e);
	setColumnWidth(0,childrenRect().width());
}

//----------------------------------------------------------------------
// KJOTSMAIN
//----------------------------------------------------------------------
KJotsMain::KJotsMain(const char* name)
  : KMainWindow( 0, name ),
    currentFolder(0),
    folderOpen(false),
    unique_id(0),
    mOptionDialog(0),
    previousEntry(0)
{
  //
  // Main widget
  //
  QWidget* f_main = new QFrame( this, "Frame_0" );
  setCentralWidget(f_main);

  QSplitter* splitter = new QSplitter(f_main);

  // the subject list
  subjectList = new MyListView(splitter, "subjectList");
  subjectList->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding));
  subjectList->setMinimumWidth(subjectList->fontMetrics().maxWidth() * 10 + 5);
  subjectList->addColumn(i18n("Pages"));
  subjectList->setColumnWidthMode(0,QListView::Manual);
  subjectList->setHScrollBarMode(QScrollView::AlwaysOff);
  subjectList->setDefaultRenameAction(QListView::Accept);
  subjectList->setSorting(-1);
  subjectList->setAlternateBackground(QColor());
  //subjectList->header()->hide();
  subjectList->setDragEnabled(true);
  subjectList->setAcceptDrops(true);
  subjectList->setItemsMovable(true);
  connect(subjectList, SIGNAL(selectionChanged(QListViewItem*)),
          this, SLOT(jumpToEntry(QListViewItem*)));
  connect(subjectList, SIGNAL(contextMenu(KListView*, QListViewItem*, const QPoint&)),
          this, SLOT(showListviewContextenu(KListView*, QListViewItem*, const QPoint&)));

  me_text = new MyMultiEdit(splitter, "me_text");
  me_text->setFocusPolicy(QWidget::StrongFocus);
  me_text->setEnabled(false);
  connect(me_text, SIGNAL(findSuccessful()), this, SLOT(slotFindSuccessful()));

  // create the hotlist
  QPushButton tp(0, "Test Button");
  bg_top = new QButtonGroup(i18n("Hotlist"), f_main, "ButtonGroup_2");
  bg_top->setFrameStyle( QFrame::Box | QFrame::Sunken );
  bg_top->setExclusive(true);
  bg_top->setMinimumHeight( tp.sizeHint().height() + 6 );
  bg_top->hide(); // don't need to see it until we add a book to the hotlist

  // set up the layout for the hotlist
  QBoxLayout* tempLayout = new QHBoxLayout(bg_top);
  bg_top_layout = new QHBoxLayout(tempLayout, KDialog::spacingHint());
  bg_top_layout->setMargin(KDialog::marginHint());
  tempLayout->addItem(new QSpacerItem(30, 20, QSizePolicy::Expanding, QSizePolicy::Minimum ));

  QVBoxLayout *bookGrid = new QVBoxLayout(f_main, KDialog::marginHint(), KDialog::spacingHint());
  bookGrid->addWidget(splitter, 0, 0);
  splitter->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding, 2, 1));
  bookGrid->addWidget(bg_top);

  button_list.setAutoDelete(true);

  connect( bg_top, SIGNAL(clicked(int)), this, SLOT(openFolder(int)) );

  updateConfiguration();

  // TODO: CTRL+Key_M was move focus, should now be rename page? erk
  // create menu
  KAction *action = KStdAction::action(KStdAction::New, this, SLOT(newEntry()), actionCollection());
  action->setText(i18n("&New Page..."));

  action = new KAction(i18n("New &Book"), "contents", CTRL+SHIFT+Key_N, this,
                       SLOT(createFolder()), actionCollection(), "new_page");
  action = KStdAction::action(KStdAction::Save, this, SLOT(saveFolder()), actionCollection());

  action->setText(i18n("Save Current Book"));
  action = new KAction(i18n("Save Book to ASCII File..."), QString::null, 0, this,
                       SLOT(writeBook()), actionCollection(), "save_book_to_ascii");
  action = new KAction(i18n("Save Page to ASCII File..."), QString::null, 0, this,
                       SLOT(writePage()), actionCollection(), "save_page_to_ascii");

  action = new KAction(i18n("Delete Current Book"), "editdelete", 0, this,
                       SLOT(deleteFolder()), actionCollection(), "del_folder");

  action = new KAction(i18n("&Delete Page"), "editdelete", CTRL+Key_D, this,
                       SLOT(deleteEntry()), actionCollection(), "del_page");

  action = KStdAction::action(KStdAction::Print, this, SLOT(slotPrintPage()), actionCollection());
  action = KStdAction::action(KStdAction::Quit, this, SLOT(slotQuit()), actionCollection());

  action = KStdAction::action(KStdAction::Cut, me_text, SLOT(cut()), actionCollection());
  connect(me_text, SIGNAL(copyAvailable(bool)), action, SLOT(setEnabled(bool)));
  action->setEnabled(false);
  action  = KStdAction::action(KStdAction::Copy, me_text, SLOT(copy()), actionCollection());
  connect(me_text, SIGNAL(copyAvailable(bool)), action, SLOT(setEnabled(bool)));
  action->setEnabled(false);
  action = new KAction(i18n("Copy &into Page Title"), "editcopy", CTRL+Key_T, this,
                       SLOT(copySelection()), actionCollection(), "copyIntoTitle");
  connect(me_text, SIGNAL(copyAvailable(bool)), action, SLOT(setEnabled(bool)));
  action->setEnabled(false);
  action  = KStdAction::action(KStdAction::Paste, me_text, SLOT(paste()), actionCollection());

  action = KStdAction::find( this, SLOT( slotSearch() ), actionCollection() );
  action = KStdAction::findNext( this, SLOT( slotRepeatSearch() ), actionCollection() );
  action->setEnabled(false);
  action = KStdAction::replace( this, SLOT( slotReplace() ), actionCollection() );

  action = new KAction(i18n("Rename Book..."), KShortcut(), this,
                       SLOT(slotEditBookTitle()), actionCollection(), "edit_folderName");
  action = new KAction(i18n("Insert Date"), "date", CTRL+Key_I, this,
                       SLOT(insertDate()), actionCollection(), "insert_date");

  action = KStdAction::prior(this, SLOT(prevEntry()), actionCollection());
  action = KStdAction::next(this, SLOT(nextEntry()), actionCollection());

  action = KStdAction::showToolbar(this, SLOT(slotShowHideToolbars()), actionCollection());
  action = KStdAction::keyBindings(this, SLOT(configureKeys()), actionCollection());
  action = KStdAction::configureToolbars(this, SLOT(configureToolbars()), actionCollection());
  action = KStdAction::preferences(this, SLOT(configure()), actionCollection());
  action = new KToggleAction(i18n("Add Current Book to Hotlist"), "launch", 0, this,
                       SLOT(addToHotlist()), actionCollection(), "add_rem_hotlist");
  createGUI();

  //
  // Set startup size.
  //
  KConfig *config = kapp->config();
  config->setGroup("kjots");


  // read hotlist
  hotlist = config->readListEntry( "Hotlist" );
  while( hotlist.count() > HOT_LIST_SIZE )
    hotlist.remove(hotlist.last());
  // read list of folders
  QDir dir(locateLocal("appdata", ""));
  QString absPath = dir.absPath() + '/';
  QStringList files = dir.entryList(QDir::Files |  QDir::Readable); // config->readListEntry( "Folders" );

  // create hotlist buttons and hotlist menu
  folders_menu = static_cast<QPopupMenu*>(guiFactory()->container("books", this));
  folders_menu->setCheckable(true);

  QPushButton *temp_button;
  for ( QStringList::Iterator it = files.begin();
        it != files.end();
        ++it)
  {
    QFile folder(absPath + *it);
    if(!folder.open(IO_ReadWrite))
    {
      continue;
    }

    QTextStream st( (QIODevice *) &folder);
    QString buf = st.readLine();

    if(buf.left(9) != "\\NewEntry")
    {
      continue;
    }

    QString decodedFilename = QFile::decodeName((*it).latin1());
    folder_list.push_back(decodedFilename);
    folders_menu->insertItem(decodedFilename, unique_id);
    if(hotlist.contains(decodedFilename))
    {
      temp_button = new QPushButton(bg_top);
      temp_button->setText(KStringHandler::rEmSqueeze(decodedFilename,
                                                     temp_button->fontMetrics(),
                                                     BUTTON_WIDTH));
      temp_button->setToggleButton(true);
      bg_top->insert(temp_button, unique_id);
      bg_top_layout->addWidget(temp_button);
      button_list.append(temp_button);
      bg_top->show();
    }

    ++unique_id;
  }

  connect( folders_menu, SIGNAL(activated(int)), this, SLOT(openFolder(int)) );
  connect( bg_top, SIGNAL(clicked(int)), this, SLOT(openFolder(int)) );

  updateConfiguration();

  int nr = folder_list.findIndex(QFile::decodeName(config->readEntry("LastOpenFolder").latin1()));
  if (nr > -1)
  {
    openFolder(nr);
  }
  else if (folder_list.at(0) != folder_list.end())
  {
    openFolder(0);
  }

  updateMenu();

  int w = config->readNumEntry("Width", 0);
  int h = config->readNumEntry("Height", 0);
  if (w == 0)
  {
    w = 600;
  }
  if (h == 0)
  {
    h = 400;
  }
  resize(w, h);

  QTimer::singleShot(0, this, SLOT(slotSetupInterface()));
}

void KJotsMain::slotSetupInterface()
{
  static_cast<KToggleAction*>(actionCollection()->action("options_show_toolbar"))->setChecked(toolBar()->isVisible());

  if (folder_list.count() == 0)
  {
    createFolder();
  }
}



KJotsMain::~KJotsMain()
{
  delete mOptionDialog;
  saveProperties();
}


void KJotsMain::saveProperties( void )
{
  KConfig* config = kapp->config();
  config->setGroup("kjots");
  button_list.clear();
  if( folderOpen )
  {
    QFileInfo fi(current_folder_name);
    config->writeEntry("LastOpenFolder", fi.fileName());
  }
  saveFolder();
  config->writeEntry("Width", width());
  config->writeEntry("Height", height());
  toolBar(0)->saveSettings(config, "MainToolbar");
  config->sync();
}

int KJotsMain::readFile( QString name )
{
  subjectList->clear();
  subjectList->setUpdatesEnabled(false);

  QFile folder(name);
  if( !folder.open(IO_ReadWrite) )
  {
    subjectList->setUpdatesEnabled(true);
    return -1;
  }

  QTextStream st( (QIODevice *) &folder);
  QString buf = st.readLine();
  if( buf.left(9) != "\\NewEntry" )
  {
    subjectList->setUpdatesEnabled(true);
    return -1;
  }

  QString subj = buf.mid(10, buf.length());
  QString body;
  KJotsEntry *entry = new KJotsEntry(subjectList, subj.isNull() ? QString::null : QString(subj));
  while( !st.eof() )
  {
    buf = st.readLine();
    if( buf.left(9) == "\\NewEntry" )
    {
      entry->setBody(body);
      body = QString::null;
      subj = buf.mid(10, buf.length() );
      entry = new KJotsEntry(subjectList, subj.isNull() ? (QString) "" : (QString) subj, entry);
      buf = st.readLine();
    }

    int pos = 0;
    while( (pos = buf.find( '\\', pos )) != -1 )
    {
      if( buf[++pos] == '\\' )
      {
        buf.remove( pos, 1 );
      }
    }

    body.append(buf + "\n");
  }

  entry->setBody(body);
  folder.close();

  updateMenu();
  subjectList->setUpdatesEnabled(true);
  return 0;
}

int KJotsMain::writeFile( QString name )
{
  // TODO: use dynamic casting instead, to allow for non-KJotsEntries
  QFile folder(name);
  if( !folder.open(IO_WriteOnly | IO_Truncate) )
    return -1;
  QTextStream st( (QIODevice *) &folder);

  bool foundPage = false;
  for(QListViewItemIterator it(subjectList);
      it.current();
      ++it)
  {
    KJotsEntry *entry = dynamic_cast<KJotsEntry*>(it.current());
    if (entry)
    {
      foundPage = true;
      st << "\\NewEntry " << entry->subject() << "\n";
      QString buf = entry->body();
      st << buf.replace('\\', "\\\\");
      if( buf.right(1) != "\n" )
      {
          st << '\n';
      }
    }
  }

  if (!foundPage)
  {
    st << "\\NewEntry \n\n";
  }

  folder.close();
  return 0;
}

void KJotsMain::openFolder(int id)
{
  if (folders_menu->indexOf(id) < 0)
  {
    return;
  }

  QPushButton *but;
  for( but = button_list.first(); but != NULL; but = button_list.next() )
    but->setOn(false);
  but = static_cast<QPushButton *>(bg_top->find(id));
  if( but )
    but->setOn(true);

  folders_menu->setItemChecked(folders_menu->idAt(currentFolder), false);
  folders_menu->setItemChecked(id, true);
  currentFolder = folders_menu->indexOf(id);
  QDir dir = QDir( locateLocal("appdata", "") );
  QString file_name = dir.absPath();
  file_name += '/';
  file_name += QFile::encodeName(*folder_list.at(currentFolder));
  if( current_folder_name == file_name )
    return;

  if (folderOpen)
  {
    saveFolder();
    previousEntry = 0;
  }
  current_folder_name = file_name;
  setCaption(kapp->makeStdCaption(*folder_list.at(currentFolder)));

  if (readFile(current_folder_name) < 0)
  {
    folderOpen = false;
    kdDebug() << "Unable to open folder" << endl;
    return;
  }

  previousEntry = 0;
  KJotsEntry* firstEntry = dynamic_cast<KJotsEntry*>(subjectList->firstChild());
  // NOTE: firstEntry is not checked for 0L yet (not necessary).  if you deref
  //       it, make sure to test it.
  subjectList->setSelected(firstEntry, true);
  prepTextArea(firstEntry);
  me_text->setEnabled(true);
  me_text->setFocus();
  folderOpen = true;

  updateMenu();
}


void KJotsMain::createFolder()
{
  KLineEditDlg dlg(i18n("Book name:"), QString::null, this);
  dlg.setCaption(i18n("New Book"));
  if (!dlg.exec())
    return;
  QString name = dlg.text();

  if( folder_list.contains(name) )
  {
    QString msg = i18n("A book with this name already exists.");
    KMessageBox::sorry( topLevelWidget(), msg );
    return;
  }
  saveFolder();
  subjectList->clear();
  folderOpen = true;
  me_text->setEnabled(true);
  me_text->setFocus();
  me_text->clear();
  me_text->removeSelection();
  new KJotsEntry(subjectList, QString::null);
  previousEntry = 0;
  subjectList->setSelected(subjectList->firstChild(), true);
  folder_list.append(name);

  ++unique_id;
  folders_menu->setItemChecked(folders_menu->idAt(currentFolder), false);
  folders_menu->insertItem(name, currentFolder);
  currentFolder = folders_menu->indexOf(unique_id);
  folders_menu->setItemChecked(unique_id, true);
  QDir dir = QDir( locateLocal("appdata", "") );
  current_folder_name = dir.absPath() + '/' + QString(QFile::encodeName(name));
  setCaption(kapp->makeStdCaption(name));
  QPushButton *but;
  for( but = button_list.first(); but != 0; but = button_list.next() )
    but->setOn(false);

  updateMenu();
}

void KJotsMain::deleteFolder()
{
  if( !folderOpen )
    return;

  QString msg = i18n("Are you sure you want to delete the current book?");
  int result = KMessageBox::warningYesNo( topLevelWidget(), msg, i18n("Delete Book") );
  if( result != KMessageBox::Yes )
  {
    return;
  }

  QFileInfo fi(current_folder_name);
  QDir dir = fi.dir(true);
  QString name = fi.fileName();
  int index = folder_list.findIndex(name);
  if( index < 0 )
    return;
  dir.remove(current_folder_name);
  folder_list.remove(folder_list.at(index));
  int id = folders_menu->idAt(index);
  folders_menu->removeItemAt(index);
  if( hotlist.contains(name) )
    {
      hotlist.remove(name);
      QButton *but = bg_top->find(id);
      bg_top->remove(but);
      button_list.remove( (QPushButton *) but );
      resize(size());
    }
  KConfig *config = KApplication::kApplication()->config();
  config->setGroup("kjots");
  config->writeEntry( "Folders", folder_list );
  config->writeEntry( "Hotlist", hotlist );
  config->sync();
  subjectList->clear();
  current_folder_name = "";
  currentFolder = 0;
  folderOpen = false;
  me_text->setEnabled(false);
  me_text->clear();
  me_text->removeSelection();
  setCaption(kapp->makeStdCaption(QString::null));
  openFolder(folders_menu->idAt(0));
  updateMenu();
}

void KJotsMain::saveFolder()
{
  if( !folderOpen )
    return;

  KJotsEntry* current = dynamic_cast<KJotsEntry*>(subjectList->currentItem());
  if (current)
  {
    current->setBody(me_text->text());
  }
  writeFile(current_folder_name);
}

void KJotsMain::nextEntry()
{
  if( !folderOpen )
    return;

  QListViewItem* nextItem = subjectList->currentItem();

  if (!nextItem)
  {
    nextItem = subjectList->firstChild();
  }
  else
  {
    nextItem = nextItem->nextSibling();
  }

  if( !nextItem )
    return;

  subjectList->setSelected(nextItem, true);
  prepTextArea(static_cast<KJotsEntry*>(nextItem));

  updateMenu();
}

void KJotsMain::prevEntry()
{
  if( !folderOpen )
    return;

  QListViewItem* prevItem = subjectList->firstChild();
  while (prevItem && prevItem->nextSibling() != subjectList->currentItem())
  {
    prevItem = prevItem->nextSibling();
  }

  if (!prevItem)
  {
    return;
  }

  subjectList->setSelected(prevItem, true);
  prepTextArea(static_cast<KJotsEntry*>(prevItem));

  updateMenu();
}

void KJotsMain::newEntry()
{
  if( !folderOpen )
    return;

  KJotsEntry* newEntry = new KJotsEntry(subjectList, QString::null, subjectList->lastChild());
  subjectList->setSelected(newEntry, true);
  prepTextArea(newEntry);

  updateMenu();
}

void KJotsMain::deleteEntry()
{
  if(!folderOpen)
    return;

  KJotsEntry* current = static_cast<KJotsEntry*>(subjectList->currentItem());
  if (!current)
    return;

  if (KMessageBox::warningYesNo(this,
                                i18n("Are you sure you want to delete the current page?"),
                                i18n("Delete Page"),
                                KStdGuiItem::yes(),
                                KStdGuiItem::no(),
                                "deletePageWarning") != KMessageBox::Yes)
  {
    return;
  }

  KJotsEntry* moveTo = static_cast<KJotsEntry*>(current->nextSibling());
  if (current == subjectList->firstChild() && !moveTo)
  {
    if (current == subjectList->firstChild())
    {
      // only one entry!
      current->setBody(QString::null);
      current->setSubject(QString::null);
      me_text->clear();
      return;
    }
  }

  delete current;

  if (!moveTo)
  {
    moveTo = static_cast<KJotsEntry*>(subjectList->lastChild());
  }

  subjectList->setSelected(moveTo, true);
  me_text->setText( moveTo->body() );

  updateMenu();
}

void KJotsMain::jumpToEntry(QListViewItem* entry)
{
  if (!folderOpen ||
      !entry ||
      entry == previousEntry)
  {
    return;
  }

  prepTextArea(static_cast<KJotsEntry*>(entry));
  updateMenu();
}

void KJotsMain::addToHotlist()
{
  if( hotlist.count() == HOT_LIST_SIZE )
    return;
  QFileInfo fi(current_folder_name);
  QString name = fi.fileName();

  if( hotlist.contains(name) )
  {
    removeFromHotlist();
    return;
  }

  hotlist.append(name);
  int index = folder_list.findIndex(name);
  if( index < 0 )
    return;
  int id = folders_menu->idAt(index);
  QPushButton *but = new QPushButton(bg_top);
  but->setText(KStringHandler::rEmSqueeze(name,
                                          but->fontMetrics(),
                                          BUTTON_WIDTH));
  button_list.append(but);
  bg_top->insert(but, id);
  bg_top_layout->addWidget(but);
  KConfig *config = kapp->config();
  config->setGroup("kjots");
  config->writeEntry( "Hotlist", hotlist );
  config->sync();

  // show the hotlist if we don't need it
  if (hotlist.count() > 0 && ! bg_top->isVisible())
  {
    bg_top->show();
  }
  but->setToggleButton(true);
  but->show();
  but->setOn(true);
  updateMenu();
}

void KJotsMain::removeFromHotlist()
{
  QFileInfo fi(current_folder_name);
  QString name = fi.fileName();
  if( !hotlist.contains(name) )
    return;
  hotlist.remove(name);
  int index = folder_list.findIndex(name);

  // is this check even necessary?
  if( index < 0 )
    return;

  int id = folders_menu->idAt(index);
  QButton *but = bg_top->find(id);
  bg_top->remove(but);
  button_list.remove( (QPushButton *) but );
  KConfig *config = kapp->config();
  config->setGroup("kjots");
  config->writeEntry( "Hotlist", hotlist );
  config->sync();

  // hide the hotlist if we don't need it
  if (button_list.count() < 1)
  {
    bg_top->hide();
  }
  updateMenu();
}

void KJotsMain::configureToolbars()
{
  saveMainWindowSettings(kapp->config(), "kjots");
  KEditToolbar dlg(actionCollection());
  connect( &dlg, SIGNAL(newToolbarConfig()), SLOT(updateToolbars()) );
  dlg.exec();
}

void KJotsMain::updateToolbars()
{
  createGUI();
  applyMainWindowSettings(kapp->config(), "kjots");
  updateMenu();
}

void KJotsMain::configure()
{
  if( mOptionDialog == 0 )
  {
    mOptionDialog = new ConfigureDialog( topLevelWidget(), 0, false );
    if( mOptionDialog == 0 ) { return; }
    connect( mOptionDialog, SIGNAL(hidden()),this,SLOT(configureHide()));
    connect( mOptionDialog, SIGNAL(valueChanged()),
             this, SLOT(updateConfiguration()) );
  }

  mOptionDialog->show();
}

void KJotsMain::configureHide()
{
  QTimer::singleShot( 0, this, SLOT(configureDestroy()) );
}

void KJotsMain::configureDestroy()
{
  if( mOptionDialog != 0 && mOptionDialog->isVisible() == false )
  {
    delete mOptionDialog; mOptionDialog = 0;
  }
}

void KJotsMain::slotSearch()
{
    me_text->search();
}

void KJotsMain::slotRepeatSearch()
{
    me_text->repeatSearch();
}

void KJotsMain::slotReplace()
{
    me_text->replace();
}

void KJotsMain::slotShowHideToolbars()
{
    if (toolBar()->isVisible())
    {
        toolBar()->hide();
    }
    else
    {
        toolBar()->show();
    }
}

void KJotsMain::updateConfiguration()
{
  KConfig* config = kapp->config();

  config->setGroup("kjots");
  QFont font( config->readEntry("EFontFamily"),
                     config->readNumEntry("EFontSize"),
                    config->readNumEntry("EFontWeight"),
                    config->readNumEntry("EFontItalic"));
  me_text->setFont(font);
  toolBar(0)->applySettings(config, "MainToolbar");
}

void KJotsMain::writeBook()
{
  saveFolder();
  QString name;
  while( name.isNull() )
  {
    KURL url = KFileDialog::getSaveURL();

    if ( url.isEmpty() )
      return;

    if ( !url.isLocalFile() )
    {
      KMessageBox::sorry( 0L, i18n( "Only local files are currently supported." ) );
      return;
    }

    name = url.path();

    QFileInfo f_info(name);
    if( f_info.exists() )
    {
      QString msg = i18n("File already exists.\n"
                         "Do you want to overwrite it?");
      int result = KMessageBox::warningYesNo( topLevelWidget(), msg );
      if( result != KMessageBox::Yes )
      {
        name = "";
      }
    }
  }
  QFile ascii_file(name);
  if( !ascii_file.open(IO_WriteOnly | IO_Truncate) )
    return;
  QTextStream st((QIODevice *) &ascii_file);

  for(QListViewItemIterator it(subjectList);
      it.current();
      ++it)
  {
    KJotsEntry* entry = dynamic_cast<KJotsEntry*>(it.current());
    if (entry)
    {
        writeEntry(st, entry);
    }
  }
  ascii_file.close();
}

void KJotsMain::writePage()
{
  saveFolder();
  QString name;
  while( name.isNull() )
  {
    KURL url = KFileDialog::getSaveURL();

    if( url.isEmpty() )
      return;

    if( !url.isLocalFile() )
    {
      KMessageBox::sorry( 0L, i18n( "Only local files are currently supported." ) );
      return;
    }

    name = url.path();

    QFileInfo f_info(name);
    if( f_info.exists() )
    {
      QString msg = i18n("File already exists.\n"
                         "Do you want to overwrite it?");
      int result = KMessageBox::warningYesNo( topLevelWidget(), msg );
      if( result != KMessageBox::Yes )
      {
        name = "";
      }
    }
  }
  QFile ascii_file(name);
  if( !ascii_file.open(IO_WriteOnly | IO_Truncate) )
    return;
  QTextStream st((QIODevice *) &ascii_file);
  writeEntry(st, static_cast<KJotsEntry*>(subjectList->currentItem()));
  ascii_file.close();
}

void KJotsMain::writeEntry(QTextStream &st, KJotsEntry *entry)
{
  if (!entry)
  {
    return;
  }

  QString line, buf;
  line.fill('#', entry->subject().length() + 2 );
  st << line << '\n';
  st << "# ";
  st << entry->subject() << '\n';
  st << line << '\n';
  buf = entry->body();
  st << buf;
  if( buf.right(1) != "\n" )
    st << '\n';
  st << '\n';
}

void KJotsMain::configureKeys()
{
  KKeyDialog::configure(actionCollection(), this);
}

void KJotsMain::copySelection()
{
  if (me_text->hasSelectedText())
  {
      KJotsEntry* current = static_cast<KJotsEntry*>(subjectList->currentItem());
      current->setSubject(me_text->selectedText());
  }
}

void KJotsMain::slotPrintPage()
{
    QString docName = current_folder_name;
    KJotsEntry* current = static_cast<KJotsEntry*>(subjectList->currentItem());
    current->setSubject(me_text->selectedText());
    if (current && !current->subject().isNull())
    {
        docName += ": " + current->subject();
    }
    me_text->print(docName);
 }

void KJotsMain::slotQuit()
{
    // just save on exit...
    saveFolder();
    close();
}

void KJotsMain::slotEditBookTitle()
{
  if (!folderOpen)
  {
    return;
  }

  bool accepted(false);
  QString currentTitle = *folder_list.at(currentFolder);
  QString newName = KLineEditDlg::getText(i18n("Rename Book"), i18n("Title of this book:"),
                                          currentTitle, &accepted, this);
  if (accepted)
  {
    QFileInfo folderInfo(current_folder_name);
    QString temp = folderInfo.dirPath(true);
    QDir dir(temp);
    temp += "/" + QFile::encodeName(newName);

    if (QFile::exists(temp))
    {
        KMessageBox::sorry(this, i18n("A book by that name already exists."), i18n("Rename Book")) ;
        return;
    }

    dir.rename(current_folder_name, temp);
    current_folder_name = temp;

    if (hotlist.contains(currentTitle))
    {
        (*hotlist.find(currentTitle)) = newName;
         KConfig *config = KApplication::kApplication()->config();
         config->setGroup("kjots");
         config->writeEntry( "Hotlist", hotlist );
         config->sync();
    }

    setCaption(kapp->makeStdCaption(newName));
    (*folder_list.at(currentFolder)) = newName;
    QPushButton* but = dynamic_cast<QPushButton*>(bg_top->find(folders_menu->idAt(currentFolder)));
    if (but)
    {
        but->setText(KStringHandler::rEmSqueeze(newName,
                                                but->fontMetrics(),
                                                BUTTON_WIDTH));
    }
  }
}

void KJotsMain::insertDate()
{
  me_text->insert(KGlobal::locale()->formatDateTime(QDateTime::currentDateTime(), true) + " ");
}

void KJotsMain::prepTextArea(KJotsEntry* entry)
{
  if (!entry)
  {
    return;
  }

  if (previousEntry)
  {
    previousEntry->setBody(me_text->text());
  }

  // remember it for the next time we come in here
  previousEntry = entry;

  me_text->setText(entry->body());
  me_text->removeSelection();
  me_text->repaint();
}

void KJotsMain::updateMenu()
{
  actionCollection()->action("del_folder")->setEnabled(folderOpen);
  actionCollection()->action("del_page")->setEnabled(folderOpen);
  actionCollection()->action("new_page")->setEnabled(folderOpen);
  actionCollection()->action("edit_folderName")->setEnabled(folderOpen);

  actionCollection()->action(KStdAction::stdName(KStdAction::Save))->setEnabled(folderOpen);
  actionCollection()->action("save_book_to_ascii")->setEnabled(folderOpen);
  actionCollection()->action("save_page_to_ascii")->setEnabled(folderOpen);

  actionCollection()->action(KStdAction::stdName(KStdAction::Paste))->setEnabled(folderOpen);

  KToggleAction* hotlistAction = static_cast<KToggleAction*>(actionCollection()->action("add_rem_hotlist"));
  hotlistAction->setEnabled(folderOpen);

  QFileInfo fi(current_folder_name);
  if (!folderOpen || (subjectList->currentItem() && !hotlist.contains(fi.fileName())))
  {
    hotlistAction->setText(i18n("Add Current Book to Hotlist"));
    hotlistAction->setChecked(false);
  }
  else
  {
    hotlistAction->setText(i18n("Remove Current Book From Hotlist"));
    hotlistAction->setChecked(true);
  }
}

void KJotsMain::slotFindSuccessful()
{
    actionCollection()->action(KStdAction::stdName(KStdAction::FindNext))->setEnabled(true);
}

void KJotsMain::renameCurrentEntry()
{
    KJotsEntry* entry = dynamic_cast<KJotsEntry*>(subjectList->currentItem());
    if (entry)
    {
        entry->editSubject();
    }
}

void KJotsMain::showListviewContextenu(KListView*, QListViewItem* i, const QPoint& p)
{
    KJotsEntry* entry = dynamic_cast<KJotsEntry*>(i);
    if (entry)
    {
        KPopupMenu* ctxMenu = new KPopupMenu(this);
        ctxMenu->insertTitle(entry->subject());
        ctxMenu->insertItem(i18n("Edit Page Title"), this, SLOT(renameCurrentEntry()));
        ctxMenu->insertItem(i18n("Delete Page"), this, SLOT(deleteEntry()));
        ctxMenu->exec(p);
        delete ctxMenu;
    }
}

#include "KJotsMain.moc"
